require('dotenv').config()
const config = require('config')
const bunyan = require('bunyan')
const Koa = require('koa')
const jsonError = require('koa-json-error')
const parser = require('koa-bodyparser')
const fs = require('fs-extra')
const consul = require('consul')(config.consul)
const koaKeycloak = require('@barath/koa-keycloak')
const healthCheck = require('koa-simple-healthcheck')
const logger = bunyan.createLogger(config.logger)
const start = async () => {
  const app = new Koa()
  app.use(parser())
  app.use(
    jsonError({
      format: err => {
        let message = err.response ? err.response.data : err.message
        const status = err.response ? err.response.status : err.status
        if (!message) message = err
        if (message.errorMessage) message = message.errorMessage
        logger.error(`${status}: ${message}`)
        try {
          message = JSON.parse(message)
        } catch (error) {}
        err.statusCode = status
        return {
          status,
          message
        }
      }
    })
  )
  app.use(healthCheck({path: '/_healthz'}))
  if (['test', 'development'].includes(process.env.NODE_ENV) && process.env.KEYCLOAK_TOKEN) {
    app.use(async (ctx, next) => {
      ctx.req.headers = {
        ...ctx.req.headers,
        authorization: `Bearer ${process.env.KEYCLOAK_TOKEN}`
      }
      await next()
    })
  }
  app.use(koaKeycloak(config.keycloak))
  fs.readdirSync('./src/routes')
    .filter(file => file !== 'index.js' && !file.includes('.test.js'))
    .map(file => file.replace('.js', ''))
    .forEach(async file => {
      const router = require(`./routes/${file}`)
      app.use(router.routes())
      app.use(router.allowedMethods())
    })
  await consul.agent.service.register(config.server)
  const server = app.listen(config.server.port, () => {
    logger.info(
      `Started ${require('../package.json').name} on port ${
        server.address().port
      }`
    )
  })
  return server
}
if (require.main === module) {
  start().catch(error => {
    logger.error(error)
  })
} else {
  module.exports = { start }
}
