const { Command, flags } = require('@oclif/command')
const fs = require('fs-extra')
const ora = require('ora')
const handlebars = require('handlebars')
const path = require('path')
const shell = require('shelljs')
const klaw = require('klaw-sync')
const presetDir = `${__dirname}/../../presets`
const exclude = ['dependencies.js']
const dirName = path.basename(path.dirname(`${process.cwd()}/package.json`))
class InitCommand extends Command {
  async run() {
    const parsed = this.parse(InitCommand)
    await this.createFiles(parsed.args.preset, parsed.flags.port)
    await this.installDependencies(parsed.args.preset)
  }
  async installDependencies(preset) {
    const spinner = ora('Installing dependencies...').start()
    const deps = require(`${presetDir}/${preset}/dependencies`)

    shell.exec('git init', { async: false, silent: true })
    shell.exec(
      `git remote add origin git@github.com:buynomics/${dirName}.git`,
      { async: false, silent: true }
    )
    shell.exec(
      `yarn add --silent ${deps.dependencies.join(' ')}`,
      {
        async: true,
        silent: true
      },
      () => {
        shell.exec(
          `yarn add --dev --silent ${deps.devDependencies.join(' ')}`,
          {
            async: true,
            silent: true
          },
          () => {
            spinner.succeed(`Initialized @brickblock/${dirName}`)
          }
        )
      }
    )
  }
  async createFiles(preset, port) {
    const spinner = ora('Creating required files...').start()
    const templates = klaw(`${presetDir}/${preset}`, {
      nodir: true
    })
      .filter(template => !exclude.includes(path.basename(template.path)))
      .map(template => template.path)
    templates.forEach(template => {
      const fileName = template.replace(
        path.resolve(`${presetDir}/${preset}`),
        process.cwd()
      )

      const content = fs.readFileSync(template, 'utf8')
      const compiled = handlebars.compile(content)
      const rendered = compiled({ pkg: dirName, port })
      const fileDir = path.dirname(fileName)
      fs.mkdirsSync(fileDir)
      fs.writeFileSync(fileName, rendered)
    })
    spinner.succeed('Created required files')
  }
}

InitCommand.description = `Initialise a Buynomics project with a preset`

InitCommand.args = [
  {
    name: 'preset',
    required: true,
    description: 'Preset to use to scaffold the project',
    options: fs.readdirSync(presetDir)
  }
]

InitCommand.flags = {
  port: flags.integer({
    char: 'p',
    description: 'Port to listen on',
    default: 3000,
    multiple: false
  })
}

module.exports = InitCommand
